<?php

/**
 * Erpal CRM Expense Report
 *
 * creates a scaled expense report with the following options:
 *
 * contrator_id => the contractor id (_erpal_basic_helper_get_own_company_nid(FALSE))
 * date_from => unix timestamp
 * date_to => unix timestamp
 * scaling => monthly or year
 * contractors => list of contractors
 * expense_categories => list of expense categories
 * dim_contractor => add contractor dimension
 * dim_expensecategory => add expense category dimension
 * no_cache => ignore caching
 *
 * @author Yann Karl
 */
class ErpalCrmReportsExpense {
  
  /**
   * create data for crm expense report, returns false if no
   * data available
   *
   * @param array $options
   * @param boolean $extended if true, the returned array will have another structure that will also contain the categories which are calculated in the method
   * @return mixed
   */    
  static function create($options = array()) {
    // flag for no result, if TRUE, query has no value (e.g. selected contractor group has no contractor)
    $noresult = FALSE;

    $expense_categories = array();
    $contractor_nids = array();
    
    $options = array_merge(self::defaults(), $options);

    //take all billables BUT not the unbillables and only billables without a parent billable
    $own_company_nid = _erpal_basic_helper_get_own_company_nid(FALSE);
    $query = new EntityFieldQuery();
    $query->entityCondition('entity_type', 'billable')   
      ->entityCondition('bundle', 'billable')
      ->propertyCondition('customer_nid', $own_company_nid, '=')
      ->propertyCondition('parent_billable', 0, '=')
      ->addMetaData('account', user_load(1)); // Run the query as user 1.

    if ($options['date_from'] > - 1) {
      $query->propertyCondition('date_delivery', $options ['date_from'], '>=');
    }

    if ($options['date_to'] > - 1) {
      $query->propertyCondition('date_delivery', $options ['date_to'], '<=');
    }
    
    if (intval(count($options['contractors'])) > 0) {
      $query->propertyCondition('contractor_nid', $options['contractors'], 'IN');
    }

    $result = $query->execute();

    if (!isset($result['billable']) || count($result['billable']) == 0) {
      $noresult = TRUE;
    }

    $retval = array('titles' => array(), 'data' => array());

    $data = array();
    $details = array();
    if ($noresult === FALSE) {
      $billable_nids = array_keys($result['billable']);
      
      foreach($billable_nids as $billable_nid) {
        $billable = billable_load($billable_nid);
        
        //no billables with parent ID, ensure this again
        if (!empty($billable->parent_billable))
          continue;
        
        $billable_w = entity_metadata_wrapper('billable', $billable);
        $currency = $billable->currency;

        $billable_date = date($options ['scaling'] == ERPAL_REPORT_DATE_SCALING_YEAR ? _erpal_reports_date_format_year() : _erpal_reports_date_format_month(), $billable->date_delivery);
        $series_title = $billable->currency;
        
        $expense_category = $billable_w->field_expense_category->value();

        if (count($options['expense_categories']) > 0 && ($expense_category == null || !in_array($expense_category->tid, $options['expense_categories']))) {         
          continue; //we only want specific expense categories that this billable does not have.
        }
        
        $expense_category_name = t('No expenses category selected');
        $expense_category_id = 0;
        if ($expense_category != NULL) {
          $expense_category_name = $expense_category->name;
          $expense_category_id = $expense_category->tid;
        }
        
        $series_title = sprintf('%s', $currency);

        if ($options ['dim_contractor']) {
          $contractor_nid = $billable->contractor_nid;
          $node_base_data = erpal_lib_helper_node_base_data(array($contractor_nid));
          $contractor = $node_base_data[$contractor_nid];
          $contractor = $contractor->title;

          $series_title = sprintf('%s - %s', $currency, $contractor);
        } elseif ($options['dim_expensecategory']) {
          $series_title = sprintf('%s : %s', $currency, $expense_category_name);
        }

        $series_title = trim($series_title);

        if (! isset($data [$series_title])) {
          $data[$series_title] = array ();
          $details[$series_title] = array ();
        }

        if (! isset($data [$series_title] [$billable_date])) {
          $data[$series_title][$billable_date] = 0;
        }
        
        $contractor_nids[$series_title][$billable_date] = $billable->contractor_nid;
        $expense_categories[$series_title] [$billable_date] = array('id' => $expense_category_id, 'name' => $expense_category_name);
        $data[$series_title][$billable_date] += $billable_w->total_price_no_vat->value();          
        
        $subject = substr($billable->subject, 0, 50); //limit billable subject to 50 chars, the json string must not be longer than 255 in total
        $details[$series_title][$billable_date][] = array('entity_type' => 'billable', 'id' => $billable->billable_id, 'title' => $subject, 'date' => $billable_date, 'value' => $billable_w->total_price_no_vat->value());
      }                     

    } else {
      $data = FALSE;
    }

    if (is_array($data) && count($data) == 0) {
      $data = false;
    }

    //build extended report data
    return array(
      'data' => $data,
      'extended' => array('expense_categories' => $expense_categories, 'contractor_nids' => $contractor_nids),
      'details' => $details,
    );
  }

  /**
   * get default options for crm expense report creation
   *
   * @return array
   */
  static function defaults() {
    return array (
        'contrator_id' => _erpal_basic_helper_get_own_company_nid(FALSE),
        'date_from' => -1,
        'date_to' => -1,
        'scaling' => ERPAL_REPORT_DATE_SCALING_MONTH,
        'contractors' => array (),        
        'expense_categories' => array (),
        'dim_contractor' => FALSE,
        'dim_contractorcategory' => FALSE,
        'dim_expensecategory' => FALSE,
        'no_cache' => FALSE
    );
  }

  /**
   * load cached erpal report object for crm expense report from database
   * if none available or dirty, new object is created and stored
   *
   * @param array $options
   * @return array
   */
  static function load($options = array()) {
    $options = array_merge(self::defaults(), $options);

    // try to load entity with given options from db, check if dirty and return cached data if not
    $query = new EntityFieldQuery();
    $query->entityCondition('entity_type', 'erpal_reports')
    ->entityCondition('bundle', 'erpal_crm_expense_reports')
    ->fieldCondition('field_erpal_reports_date_from', 'value', $options['date_from'], '=')
    ->fieldCondition('field_erpal_reports_date_to', 'value', $options['date_to'], '=')
    ->fieldCondition('field_erpal_reports_date_scaling', 'value', $options['scaling'], '=')
    ->fieldCondition('field_erp_crm_exp_rep_dim_cont', 'value', $options['dim_contractor'] === TRUE ? 1 : 0, '=')
    ->fieldCondition('field_erp_crm_exp_rep_dim_expcat', 'value', $options['dim_expensecategory'] === TRUE ? 1 : 0, '=');

    if (count($options['contractors']) > 0) {
      $query->fieldCondition('field_contractor_ref', 'target_id', $options['contractors'], 'IN');
    }
    if (count($options['expense_categories']) > 0) {
      $query->fieldCondition('field_expense_category', 'tid', $options['expense_categories'], 'IN');
    }

    $result = $query->execute();

    $entity = NULL;
    if (count($result) > 0) {
      $entity = entity_load_single('erpal_reports', key($result ['erpal_reports']));
      $entity_wrapper = entity_metadata_wrapper('erpal_reports', $entity);

      if ($options['no_cache'] == FALSE && $entity_wrapper->dirty->value() == 0) {
        $ret = (array)json_decode($entity_wrapper->data->value());        
        $ret['report_id'] = $entity->id;        
        return $ret;        
      }
    }

    $retval = self::create($options);

    if ($entity == NULL) {
      $entity = entity_create('erpal_reports', array('type' =>'erpal_crm_expense_reports'));
    }

    // store data to entity, reset dirty flag
    $entity->dirty = 0;
    $entity->field_erpal_reports_date_from = array(LANGUAGE_NONE => array(0 => array('value' => $options['date_from'])));
    $entity->field_erpal_reports_date_to = array(LANGUAGE_NONE => array(0 => array('value' => $options['date_to'])));
    $entity->field_erpal_reports_date_scaling = array(LANGUAGE_NONE => array(0 => array('value' => $options['scaling'])));
    $entity->field_erp_crm_exp_rep_dim_cont = array(LANGUAGE_NONE => array(0 => array('value' => $options['dim_contractor'] === TRUE ? 1 : 0)));
    $entity->field_erp_crm_exp_rep_dim_expcat = array(LANGUAGE_NONE => array(0 => array('value' => $options['dim_expensecategory'] === TRUE ? 1 : 0)));

    if (count($options['contractors']) > 0) {
      $contractors = array();
      foreach($options['contractors'] as $contractor) {
        $contractors[] = array('target_id' => $contractor);
      }
      $entity->field_contractor_ref = array(LANGUAGE_NONE => $contractors);
    }

    if (count($options['expense_categories']) > 0) {
      $expense_categories = array();
      foreach($options['expense_categories'] as $expense_category) {
        $expense_categories[] = array('tid' => $expense_category);
      }
      $entity->field_expense_category = array(LANGUAGE_NONE => $expense_categories);
    }

    $entity->save();
    $retval['report_id'] = $entity->id;
    
    //save details first and remove this big data
    _erpal_reports_save_details($entity, $retval, TRUE);
    //$entity->data = json_encode($retval);
    
    return $retval;
  }
}
